@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: bacon-video-widget-gst.c,v 1.160 2005/03/18 11:35:35 rbultje Exp $
+ * $Id: bacon-video-widget-gst.c,v 1.160.2.1 2005/04/18 09:03:09 rbultje Exp $
  *
  *
  * The Totem project hereby grant permission for non-gpl compatible GStreamer
@@ -2143,7 +2143,7 @@ bacon_video_widget_set_show_visuals (BaconVideoWidget * bvw,
 
   bvw->priv->show_vfx = show_visuals;
   gconf_client_set_bool (bvw->priv->gc,
-      GCONF_PREFIX"/enable_visualization", TRUE, NULL);
+      GCONF_PREFIX"/enable_visualization", show_visuals, NULL);
 
   setup_vis (bvw);
 
