@@ -1,7 +1,7 @@
 /* 
    arch-tag: Implementation of Rhythmbox playlist parser
 
-   Copyright (C) 2002, 2003 Bastien Nocera
+   Copyright (C) 2002, 2003, 2004 Bastien Nocera
    Copyright (C) 2003,2004 Colin Walters <walters@rhythmbox.org>
 
    The Gnome Library is free software; you can redistribute it and/or
@@ -22,25 +22,29 @@
    Author: Bastien Nocera <hadess@hadess.net>
  */
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif /* HAVE_CONFIG_H */
+
 #include "totem-pl-parser.h"
 
 #include "totemplparser-marshal.h"
+#include "totem-disc.h"
 
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <libxml/tree.h>
 #include <libxml/parser.h>
-#include <glade/glade.h>
-#include <gconf/gconf-client.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
 #include <libgnomevfs/gnome-vfs-mime-utils.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
 #include <string.h>
 
 #define READ_CHUNK_SIZE 8192
 #define MIME_READ_CHUNK_SIZE 1024
+#define RECURSE_LEVEL_MAX 4
 
 typedef TotemPlParserResult (*PlaylistCallback) (TotemPlParser *parser, const char *url, gpointer data);
 static gboolean totem_pl_parser_scheme_is_ignored (TotemPlParser *parser, const char *url);
@@ -178,6 +182,9 @@ totem_pl_parser_base_url (const char *url)
 	char *base;
 
 	uri = gnome_vfs_uri_new (url);
+	if (uri == NULL)
+		return NULL;
+
 	parent = gnome_vfs_uri_get_parent (uri);
 	if (!parent) {
 		parent = uri;
@@ -216,7 +223,7 @@ write_string (GnomeVFSHandle *handle, const char *buf, GError **error)
 
 static int
 totem_pl_parser_num_entries (TotemPlParser *parser, GtkTreeModel *model,
-			     TotemPlParserIterFunc func)
+			     TotemPlParserIterFunc func, gpointer user_data)
 {
 	int num_entries, i, ignored;
 
@@ -232,7 +239,7 @@ totem_pl_parser_num_entries (TotemPlParser *parser, GtkTreeModel *model,
 		gtk_tree_model_get_iter_from_string (model, &iter, path);
 		g_free (path);
 
-		func (model, &iter, &url, &title);
+		func (model, &iter, &url, &title, user_data);
 		if (totem_pl_parser_scheme_is_ignored (parser, url) != FALSE)
 			ignored++;
 
@@ -251,6 +258,9 @@ totem_pl_parser_relative (const char *url, const char *output)
 
 	base = NULL;
 	url_base = totem_pl_parser_base_url (url);
+	if (url_base == NULL)
+		return NULL;
+
 	output_base = totem_pl_parser_base_url (output);
 
 	needle = strstr (url_base, output_base);
@@ -283,8 +293,8 @@ totem_pl_parser_relative (const char *url, const char *output)
 
 static gboolean
 totem_pl_parser_write_pls (TotemPlParser *parser, GtkTreeModel *model,
-		TotemPlParserIterFunc func, const char *output,
-		GError **error)
+                           TotemPlParserIterFunc func, 
+                           const char *output, gpointer user_data, GError **error)
 {
 	GnomeVFSHandle *handle;
 	GnomeVFSResult res;
@@ -292,7 +302,7 @@ totem_pl_parser_write_pls (TotemPlParser *parser, GtkTreeModel *model,
 	char *buf;
 	gboolean success;
 
-	num_entries = totem_pl_parser_num_entries (parser, model, func);
+	num_entries = totem_pl_parser_num_entries (parser, model, func, user_data);
 	num_entries_total = gtk_tree_model_iter_n_children (model, NULL);
 
 	res = gnome_vfs_open (&handle, output, GNOME_VFS_OPEN_WRITE);
@@ -336,7 +346,7 @@ totem_pl_parser_write_pls (TotemPlParser *parser, GtkTreeModel *model,
 		gtk_tree_model_get_iter_from_string (model, &iter, path);
 		g_free (path);
 
-		func (model, &iter, &url, &title);
+		func (model, &iter, &url, &title, user_data);
 
 		if (totem_pl_parser_scheme_is_ignored (parser, url) != FALSE)
 		{
@@ -410,7 +420,7 @@ totem_pl_parser_url_to_dos (const char *url, const char *output)
 static gboolean
 totem_pl_parser_write_m3u (TotemPlParser *parser, GtkTreeModel *model,
 		TotemPlParserIterFunc func, const char *output,
-		gboolean dos_compatible, GError **error)
+		gboolean dos_compatible, gpointer user_data, GError **error)
 {
 	GnomeVFSHandle *handle;
 	GnomeVFSResult res;
@@ -446,7 +456,7 @@ totem_pl_parser_write_m3u (TotemPlParser *parser, GtkTreeModel *model,
 		gtk_tree_model_get_iter_from_string (model, &iter, path);
 		g_free (path);
 
-		func (model, &iter, &url, &title);
+		func (model, &iter, &url, &title, user_data);
 
 		if (totem_pl_parser_scheme_is_ignored (parser, url) != FALSE)
 		{
@@ -485,18 +495,18 @@ totem_pl_parser_write_m3u (TotemPlParser *parser, GtkTreeModel *model,
 gboolean
 totem_pl_parser_write (TotemPlParser *parser, GtkTreeModel *model,
 		TotemPlParserIterFunc func, const char *output,
-		TotemPlParserType type, GError **error)
+		TotemPlParserType type, gpointer user_data, GError **error)
 {
 	switch (type)
 	{
 	case TOTEM_PL_PARSER_PLS:
 		return totem_pl_parser_write_pls (parser, model, func,
-				output, error);
+				output, user_data, error);
 	case TOTEM_PL_PARSER_M3U:
 	case TOTEM_PL_PARSER_M3U_DOS:
 		return totem_pl_parser_write_m3u (parser, model, func,
 				output, (type == TOTEM_PL_PARSER_M3U_DOS),
-				error);
+                                user_data, error);
 	default:
 		g_assert_not_reached ();
 	}
@@ -504,65 +514,6 @@ totem_pl_parser_write (TotemPlParser *parser, GtkTreeModel *model,
 	return FALSE;
 }
 
-static GnomeVFSResult
-my_eel_read_entire_file (const char *uri,
-		int *file_size,
-		char **file_contents)
-{
-	GnomeVFSResult result;
-	GnomeVFSHandle *handle;
-	char *buffer;
-	GnomeVFSFileSize total_bytes_read;
-	GnomeVFSFileSize bytes_read;
-
-	*file_size = 0;
-	*file_contents = NULL;
-
-	/* Open the file. */
-	result = gnome_vfs_open (&handle, uri, GNOME_VFS_OPEN_READ);
-	if (result != GNOME_VFS_OK) {
-		return result;
-	}
-
-	/* Read the whole thing. */
-	buffer = NULL;
-	total_bytes_read = 0;
-	do {
-		buffer = g_realloc (buffer, total_bytes_read + READ_CHUNK_SIZE);
-		result = gnome_vfs_read (handle,
-				buffer + total_bytes_read,
-				READ_CHUNK_SIZE,
-				&bytes_read);
-		if (result != GNOME_VFS_OK && result != GNOME_VFS_ERROR_EOF) {
-			g_free (buffer);
-			gnome_vfs_close (handle);
-			return result;
-		}
-
-		/* Check for overflow. */
-		if (total_bytes_read + bytes_read < total_bytes_read) {
-			g_free (buffer);
-			gnome_vfs_close (handle);
-			return GNOME_VFS_ERROR_TOO_BIG;
-		}
-
-		total_bytes_read += bytes_read;
-	} while (result == GNOME_VFS_OK);
-
-	/* Close the file. */
-	result = gnome_vfs_close (handle);
-	if (result != GNOME_VFS_OK) {
-		g_free (buffer);
-		return result;
-	}
-
-	/* Return the file. */
-	*file_size = total_bytes_read;
-	*file_contents = g_realloc (buffer, total_bytes_read);
-
-	return GNOME_VFS_OK;
-}
-
 static int
 read_ini_line_int (char **lines, const char *key)
 {
@@ -639,25 +590,37 @@ totem_pl_parser_finalize (GObject *object)
 	g_return_if_fail (parser->priv != NULL);
 
 	g_list_foreach (parser->priv->ignore_schemes, (GFunc) g_free, NULL);
+	g_free (parser->priv);
+	parser->priv = NULL;
 
 	if (G_OBJECT_CLASS (parent_class)->finalize != NULL) {
 		(* G_OBJECT_CLASS (parent_class)->finalize) (object);
 	}
 }
 
+static gboolean
+totem_pl_parser_check_utf8 (const char *title)
+{
+	return title ? g_utf8_validate (title, -1, NULL) : FALSE;
+}
+
 static void
 totem_pl_parser_add_one_url (TotemPlParser *parser, const char *url, const char *title)
 {
 	g_signal_emit (G_OBJECT (parser), totem_pl_parser_table_signals[ENTRY],
-		       0, url, title, NULL);
+		       0, url,
+		       totem_pl_parser_check_utf8 (title) ? title : NULL,
+		       NULL);
 }
 
 static void
-totem_pl_parser_add_one_url_ext (TotemPlParser *parser, const char *url, const char *title,
-			     const char *genre)
+totem_pl_parser_add_one_url_ext (TotemPlParser *parser, const char *url,
+		const char *title, const char *genre)
 {
 	g_signal_emit (G_OBJECT (parser), totem_pl_parser_table_signals[ENTRY],
-		       0, url, title, genre);
+		       0, url,
+		       totem_pl_parser_check_utf8 (title) ? title : NULL,
+		       genre);
 }
 
 static TotemPlParserResult
@@ -668,7 +631,7 @@ totem_pl_parser_add_ram (TotemPlParser *parser, const char *url, gpointer data)
 	int size, i;
 	const char *split_char;
 
-	if (my_eel_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
+	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 
 	contents = g_realloc (contents, size + 1);
@@ -752,7 +715,7 @@ totem_pl_parser_add_m3u (TotemPlParser *parser, const char *url, gpointer data)
 	const char *split_char;
 	gboolean extinfo;
 
-	if (my_eel_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
+	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
 		return FALSE;
 
 	contents = g_realloc (contents, size + 1);
@@ -834,7 +797,7 @@ totem_pl_parser_add_asf_parser (TotemPlParser *parser, const char *url,
 	char *contents, **lines, *ref;
 	int size;
 
-	if (my_eel_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
+	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 
 	contents = g_realloc (contents, size + 1);
@@ -872,7 +835,7 @@ totem_pl_parser_add_pls (TotemPlParser *parser, const char *url, gpointer data)
 	char *split_char;
 	gboolean dos_mode = FALSE;
 
-	if (my_eel_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
+	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 
 	if (size == 0)
@@ -924,9 +887,34 @@ totem_pl_parser_add_pls (TotemPlParser *parser, const char *url, gpointer data)
 		g_free (title_key);
 		g_free (genre_key);
 
-		if (file != NULL)
+		if (file == NULL)
+		{
+			g_free (file);
+			g_free (title);
+			g_free (genre);
+			continue;
+		}
+
+		if (strstr (file, "://") != NULL
+				|| file[0] == G_DIR_SEPARATOR) {
 			totem_pl_parser_add_one_url_ext (parser,
 					file, title, genre);
+		} else {
+			char *uri, *base, *escaped;
+
+			/* Try with a base */
+			base = totem_pl_parser_base_url (url);
+			escaped = gnome_vfs_escape_path_string (file);
+
+			uri = g_strdup_printf ("%s/%s", base, escaped);
+
+			totem_pl_parser_add_one_url_ext (parser,
+					uri, title, genre);
+
+			g_free (escaped);
+			g_free (uri);
+			g_free (base);
+		}
 
 		g_free (file);
 		g_free (title);
@@ -1021,6 +1009,11 @@ parse_asx_entries (TotemPlParser *parser, char *base, xmlDocPtr doc,
 			if (parse_asx_entry (parser, base, doc, parent, title) != FALSE)
 				retval = TRUE;
 		}
+		if (g_ascii_strcasecmp (node->name, "repeat") == 0) {
+			/* Repeat at the top-level */
+			if (parse_asx_entries (parser, base, doc, node) != FALSE)
+				retval = TRUE;
+		}
 	}
 
 	g_free (title);
@@ -1037,7 +1030,7 @@ totem_pl_parser_add_asx (TotemPlParser *parser, const char *url, gpointer data)
 	int size;
 	gboolean retval = TOTEM_PL_PARSER_RESULT_UNHANDLED;
 
-	if (my_eel_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
+	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
 		return FALSE;
 
 	contents = g_realloc (contents, size + 1);
@@ -1168,7 +1161,7 @@ totem_pl_parser_add_smil (TotemPlParser *parser, const char *url, gpointer data)
 	int size;
 	gboolean retval = TOTEM_PL_PARSER_RESULT_UNHANDLED;
 
-	if (my_eel_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
+	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 
 	contents = g_realloc (contents, size + 1);
@@ -1195,6 +1188,7 @@ totem_pl_parser_add_smil (TotemPlParser *parser, const char *url, gpointer data)
 		if (parse_smil_entries (parser, base, doc, node) != FALSE)
 			retval = TOTEM_PL_PARSER_RESULT_SUCCESS;
 
+	g_free (base);
 	xmlFreeDoc (doc);
 
 	return retval;
@@ -1205,7 +1199,8 @@ totem_pl_parser_add_asf (TotemPlParser *parser, const char *url, gpointer data)
 {
 	if (data != NULL &&
 		(g_str_has_prefix (data, "[Reference]") == FALSE
-		 && g_ascii_strncasecmp (data, "<ASX", strlen ("<ASX")) != 0)) {
+		 && g_ascii_strncasecmp (data, "<ASX", strlen ("<ASX")) != 0
+		 && strstr (data, "<ASX") == NULL)) {
 		totem_pl_parser_add_one_url (parser, url, NULL);
 		return TOTEM_PL_PARSER_RESULT_SUCCESS;
 	}
@@ -1220,7 +1215,7 @@ totem_pl_parser_add_desktop (TotemPlParser *parser, const char *url, gpointer da
 	const char *path, *display_name, *type;
 	int size;
 
-	if (my_eel_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
+	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 
 	contents = g_realloc (contents, size + 1);
@@ -1275,9 +1270,23 @@ static TotemPlParserResult
 totem_pl_parser_add_directory (TotemPlParser *parser, const char *url,
 			   gpointer data)
 {
+	MediaType type;
 	GList *list, *l;
 	GnomeVFSResult res;
 
+	if (parser->priv->recurse_level == 1) {
+		char *media_url;
+
+		type = totem_cd_detect_type_from_dir (url, &media_url, NULL);
+		if (type != MEDIA_TYPE_DATA && type != MEDIA_TYPE_ERROR) {
+			if (media_url != NULL) {
+				totem_pl_parser_add_one_url (parser, media_url, NULL);
+				g_free (media_url);
+				return TOTEM_PL_PARSER_RESULT_SUCCESS;
+			}
+		}
+	}
+
 	res = gnome_vfs_directory_list_load (&list, url,
 			GNOME_VFS_FILE_INFO_DEFAULT);
 	if (res != GNOME_VFS_OK)
@@ -1287,7 +1296,7 @@ totem_pl_parser_add_directory (TotemPlParser *parser, const char *url,
 	l = list;
 
 	while (l != NULL) {
-		char *str, *fullpath;
+		char *name, *fullpath;
 		GnomeVFSFileInfo *info = l->data;
 
 		if (info->name != NULL && (strcmp (info->name, ".") == 0
@@ -1296,16 +1305,13 @@ totem_pl_parser_add_directory (TotemPlParser *parser, const char *url,
 			continue;
 		}
 
-		str = g_build_filename (url, info->name, NULL);
-		if (strstr (str, "://") != NULL && str[0] == '/')
-			fullpath = str + 1;
-		else
-			fullpath = str;
+		name = gnome_vfs_escape_string (info->name);
+		fullpath = g_strconcat (url, "/", name, NULL);
+		g_free (name);
 
 		if (totem_pl_parser_parse_internal (parser, fullpath) != TOTEM_PL_PARSER_RESULT_SUCCESS)
 			totem_pl_parser_add_one_url (parser, fullpath, NULL);
 
-		g_free (str);
 		l = l->next;
 	}
 
@@ -1318,6 +1324,7 @@ totem_pl_parser_add_directory (TotemPlParser *parser, const char *url,
 /* These ones need a special treatment, mostly parser formats */
 static PlaylistTypes special_types[] = {
 	{ "audio/x-mpegurl", totem_pl_parser_add_m3u },
+	{ "audio/playlist", totem_pl_parser_add_m3u },
 	{ "audio/x-ms-asx", totem_pl_parser_add_asx },
 	{ "audio/x-scpls", totem_pl_parser_add_pls },
 	{ "application/x-smil", totem_pl_parser_add_smil },
@@ -1396,11 +1403,9 @@ totem_pl_parser_parse_internal (TotemPlParser *parser, const char *url)
 	gpointer data = NULL;
 	gboolean ret = FALSE;
 
-	if (parser->priv->recurse_level > 4)
+	if (parser->priv->recurse_level > RECURSE_LEVEL_MAX)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 
-	parser->priv->recurse_level++;
-
 	if (totem_pl_parser_ignore (parser, url) != FALSE)
 		return TOTEM_PL_PARSER_RESULT_UNHANDLED;
 
@@ -1411,6 +1416,8 @@ totem_pl_parser_parse_internal (TotemPlParser *parser, const char *url)
 	if (mimetype == NULL)
 		return TOTEM_PL_PARSER_RESULT_UNHANDLED;
 
+	parser->priv->recurse_level++;
+
 	for (i = 0; i < G_N_ELEMENTS(special_types); i++) {
 		if (strcmp (special_types[i].mimetype, mimetype) == 0) {
 			ret = (* special_types[i].func) (parser, url, data);
@@ -1430,6 +1437,8 @@ totem_pl_parser_parse_internal (TotemPlParser *parser, const char *url)
 		}
 	}
 
+	parser->priv->recurse_level--;
+
 	if (ret == FALSE && parser->priv->fallback) {
 		totem_pl_parser_add_one_url (parser, url, NULL);
 		return TOTEM_PL_PARSER_RESULT_SUCCESS;
@@ -1447,6 +1456,8 @@ totem_pl_parser_parse (TotemPlParser *parser, const char *url,
 {
 	g_return_val_if_fail (TOTEM_IS_PL_PARSER (parser), TOTEM_PL_PARSER_RESULT_UNHANDLED);
 	g_return_val_if_fail (url != NULL, TOTEM_PL_PARSER_RESULT_UNHANDLED);
+	g_return_val_if_fail (strstr (url, "://") != NULL,
+			TOTEM_PL_PARSER_RESULT_UNHANDLED);
 
 	parser->priv->recurse_level = 0;
 	parser->priv->fallback = fallback;
