@@ -16,6 +16,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
+ * The Totem project hereby grant permission for non-gpl compatible GStreamer
+ * plugins to be used and distributed together with GStreamer and Totem. This
+ * permission are above and beyond the permissions granted by the GPL license
+ * Totem is covered by.
+ *
+ * Monday 7th February 2005: Christian Schaller: Add excemption clause.
+ * See license_change file for details.
+ *
  */
 
 #include "config.h"
@@ -77,6 +85,8 @@
 #define ZOOM_IN_OFFSET 1
 #define ZOOM_OUT_OFFSET -1
 
+#define FULLSCREEN_POPUP_TIMEOUT 5 * 1000
+
 #define BVW_VBOX_BORDER_WIDTH 1
 
 static const GtkTargetEntry target_table[] = {
@@ -191,7 +201,7 @@ totem_action_menu_popup (Totem *totem, guint button)
 {
 	GtkWidget *menu;
 
-	menu = glade_xml_get_widget (totem->xml,
+	menu = glade_xml_get_widget (totem->xml_popup,
 			"totem_right_click_menu");
 	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
 			button, gtk_get_current_event_time ());
@@ -293,11 +303,14 @@ totem_action_play (Totem *totem)
 
 	retval = bacon_video_widget_play (totem->bvw,  &err);
 	play_pause_set_label (totem, retval ? STATE_PLAYING : STATE_STOPPED);
+	if (totem_is_fullscreen (totem) != FALSE)
+		totem_scrsaver_disable (totem->scr);
+
 	if (retval == FALSE)
 	{
 		char *msg, *disp;
 
-		disp = gnome_vfs_unescape_string_for_display (totem->mrl);
+		disp = totem_uri_escape_for_display (totem->mrl);
 		msg = g_strdup_printf(_("Totem could not play '%s'."), disp);
 		g_free (disp);
 
@@ -324,7 +337,7 @@ totem_action_seek (Totem *totem, double pos)
 	{
 		char *msg, *disp;
 
-		disp = gnome_vfs_unescape_string_for_display (totem->mrl);
+		disp = totem_uri_escape_for_display (totem->mrl);
 		msg = g_strdup_printf(_("Totem could not play '%s'."), disp);
 		g_free (disp);
 
@@ -387,6 +400,7 @@ void
 totem_action_stop (Totem *totem)
 {
 	bacon_video_widget_stop (totem->bvw);
+	totem_scrsaver_enable (totem->scr);
 }
 
 void
@@ -413,9 +427,12 @@ totem_action_play_pause (Totem *totem)
 	{
 		bacon_video_widget_play (totem->bvw, NULL);
 		play_pause_set_label (totem, STATE_PLAYING);
+		if (totem_is_fullscreen (totem) != FALSE)
+			totem_scrsaver_disable (totem->scr);
 	} else {
 		bacon_video_widget_pause (totem->bvw);
 		play_pause_set_label (totem, STATE_PAUSED);
+		totem_scrsaver_enable (totem->scr);
 	}
 }
 
@@ -430,6 +447,7 @@ totem_action_fullscreen_toggle (Totem *totem)
 		bacon_video_widget_set_fullscreen (totem->bvw, FALSE);
 		gtk_window_unfullscreen (GTK_WINDOW(totem->win));
 		bacon_video_widget_set_show_cursor (totem->bvw, TRUE);
+
 		totem_scrsaver_enable (totem->scr);
 
 		totem->controls_visibility = TOTEM_CONTROLS_VISIBLE;
@@ -452,7 +470,8 @@ totem_action_fullscreen_toggle (Totem *totem)
 		bacon_video_widget_set_fullscreen (totem->bvw, TRUE);
 		bacon_video_widget_set_show_cursor (totem->bvw, FALSE);
 		gtk_window_fullscreen (GTK_WINDOW(totem->win));
-		totem_scrsaver_disable (totem->scr);
+		if (bacon_video_widget_is_playing (totem->bvw) != FALSE)
+			totem_scrsaver_disable (totem->scr);
 
 		totem->controls_visibility = TOTEM_CONTROLS_FULLSCREEN;
 		show_controls (totem, FALSE);
@@ -594,7 +613,7 @@ totem_action_set_mrl_with_warning (Totem *totem, const char *mrl,
 		widget = glade_xml_get_widget (totem->xml, "tmw_play_menu_item");
 		gtk_widget_set_sensitive (widget, FALSE);
 
-		widget = glade_xml_get_widget (totem->xml, "trcm_play");
+		widget = glade_xml_get_widget (totem->xml_popup, "trcm_play");
 		gtk_widget_set_sensitive (widget, FALSE);
 
 		/* Seek bar and seek buttons */
@@ -607,9 +626,9 @@ totem_action_set_mrl_with_warning (Totem *totem, const char *mrl,
 		gtk_widget_set_sensitive (widget, FALSE);
 		widget = glade_xml_get_widget (totem->xml, "tmw_volume_down_menu_item");
 		gtk_widget_set_sensitive (widget, FALSE);
-		widget = glade_xml_get_widget (totem->xml, "trcm_volume_up");
+		widget = glade_xml_get_widget (totem->xml_popup, "trcm_volume_up");
 		gtk_widget_set_sensitive (widget, FALSE);
-		widget = glade_xml_get_widget (totem->xml, "trcm_volume_down");
+		widget = glade_xml_get_widget (totem->xml_popup, "trcm_volume_down");
 		gtk_widget_set_sensitive (widget, FALSE);
 
 		/* Control popup */
@@ -620,9 +639,9 @@ totem_action_set_mrl_with_warning (Totem *totem, const char *mrl,
 		gtk_widget_set_sensitive (widget, FALSE);
 		widget = glade_xml_get_widget (totem->xml, "tcw_next_button"); 
 		gtk_widget_set_sensitive (widget, FALSE);
-		widget = glade_xml_get_widget (totem->xml, "trcm_next_chapter");
+		widget = glade_xml_get_widget (totem->xml_popup, "trcm_next_chapter");
 		gtk_widget_set_sensitive (widget, FALSE);
-		widget = glade_xml_get_widget (totem->xml, "trcm_previous_chapter");
+		widget = glade_xml_get_widget (totem->xml_popup, "trcm_previous_chapter");
 		gtk_widget_set_sensitive (widget, FALSE);
 		widget = glade_xml_get_widget (totem->xml, "tcw_volume_hbox");
 		gtk_widget_set_sensitive (widget, FALSE);
@@ -657,7 +676,7 @@ totem_action_set_mrl_with_warning (Totem *totem, const char *mrl,
 		gtk_widget_set_sensitive (totem->pp_button, TRUE);
 		widget = glade_xml_get_widget (totem->xml, "tmw_play_menu_item");
 		gtk_widget_set_sensitive (widget, TRUE);
-		widget = glade_xml_get_widget (totem->xml, "trcm_play");
+		widget = glade_xml_get_widget (totem->xml_popup, "trcm_play");
 		gtk_widget_set_sensitive (widget, TRUE);
 		gtk_widget_set_sensitive (totem->fs_pp_button, TRUE);
 
@@ -675,6 +694,10 @@ totem_action_set_mrl_with_warning (Totem *totem, const char *mrl,
 		gtk_widget_set_sensitive (widget, caps);
 		widget = glade_xml_get_widget (totem->xml, "tcw_volume_hbox");
 		gtk_widget_set_sensitive (widget, caps);
+		widget = glade_xml_get_widget (totem->xml_popup, "trcm_volume_up");                gtk_widget_set_sensitive (widget, caps);
+		widget = glade_xml_get_widget (totem->xml_popup, "trcm_volume_down");
+		gtk_widget_set_sensitive (widget, caps);
+
 
 		/* Take a screenshot */
 		widget = glade_xml_get_widget (totem->xml,
@@ -688,7 +711,7 @@ totem_action_set_mrl_with_warning (Totem *totem, const char *mrl,
 		{
 			char *msg, *disp;
 
-			disp = gnome_vfs_unescape_string_for_display (totem->mrl);
+			disp = totem_uri_escape_for_display (totem->mrl);
 			msg = g_strdup_printf(_("Totem could not play '%s'."), disp);
 			g_free (disp);
 			totem_action_error (msg, err->message, totem);
@@ -793,7 +816,7 @@ totem_action_seek_relative (Totem *totem, int off_sec)
 	{
 		char *msg, *disp;
 
-		disp = gnome_vfs_unescape_string_for_display (totem->mrl);
+		disp = totem_uri_escape_for_display (totem->mrl);
 		msg = g_strdup_printf(_("Totem could not play '%s'."), totem->mrl);
 		g_free (disp);
 
@@ -1065,10 +1088,14 @@ on_recent_file_activate (EggRecentViewGtk *view, EggRecentItem *item,
 
 	uri = egg_recent_item_get_uri (item);
 
+	totem_signal_block_by_data (totem->playlist, totem);
+
 	end = totem_playlist_get_last (totem->playlist);
 	playlist_changed = totem_playlist_add_mrl (totem->playlist, uri, NULL);
 	egg_recent_model_add_full (totem->recent_model, item);
 
+	totem_signal_unblock_by_data (totem->playlist, totem);
+
 	if (playlist_changed)
 	{
 		char *mrl;
@@ -1082,9 +1109,19 @@ on_recent_file_activate (EggRecentViewGtk *view, EggRecentItem *item,
 	g_free (uri);
 }
 
+static void
+on_got_redirect (BaconVideoWidget *bvw, const char *mrl, Totem *totem)
+{
+	g_message ("on_got_redirect %s", mrl);
+	//FIXME we need to check for relative paths here
+	bacon_video_widget_close (totem->bvw);
+	bacon_video_widget_open (totem->bvw, mrl, NULL);
+	bacon_video_widget_play (bvw, NULL);
+}
+
 /* This is only called when we are playing a DVD */
 static void
-on_title_change_event (GtkWidget *win, const char *string, Totem *totem)
+on_title_change_event (BaconVideoWidget *bvw, const char *string, Totem *totem)
 {
 	update_mrl_label (totem, string);
 	totem_playlist_set_title (TOTEM_PLAYLIST (totem->playlist), string);
@@ -1161,6 +1198,8 @@ update_seekable (Totem *totem, gboolean seekable)
 	if (totem->seekable == seekable)
 		return;
 
+	totem->seekable = seekable;
+
 	/* Check if the stream is seekable */
 	gtk_widget_set_sensitive (totem->seek, seekable);
 	gtk_widget_set_sensitive (totem->fs_seek, seekable);
@@ -1177,9 +1216,10 @@ update_seekable (Totem *totem, gboolean seekable)
 	widget = glade_xml_get_widget (totem->xml,
 			"tmw_skip_backwards_menu_item");
 	gtk_widget_set_sensitive (widget, seekable);
-	widget = glade_xml_get_widget (totem->xml, "trcm_skip_forward");
+	widget = glade_xml_get_widget (totem->xml_popup, "trcm_skip_forward");
 	gtk_widget_set_sensitive (widget, seekable);
-	widget = glade_xml_get_widget (totem->xml, "trcm_skip_backwards");
+	widget = glade_xml_get_widget (totem->xml_popup,
+			"trcm_skip_backwards");
 	gtk_widget_set_sensitive (widget, seekable);
 	widget = glade_xml_get_widget (totem->xml, "tmw_skip_to_menu_item");
 	gtk_widget_set_sensitive (widget, seekable);
@@ -1534,17 +1574,18 @@ on_open_location1_activate (GtkButton *button, Totem *totem)
 {
 	GladeXML *glade;
 	char *mrl;
-	GtkWidget *dialog, *entry;
+	GtkWidget *dialog, *entry, *gentry;
 	int response;
 	const char *filenames[2];
 
-	glade = totem_interface_load ("uri.glade", "Open Location...",
+	glade = totem_interface_load ("uri.glade", _("Open Location..."),
 			FALSE, GTK_WINDOW (totem->win));
 	if (glade == NULL)
 		return;
 
 	dialog = glade_xml_get_widget (glade, "open_uri_dialog");
 	entry = glade_xml_get_widget (glade, "uri");
+	gentry = glade_xml_get_widget (glade, "uri_list");
 
 	response = gtk_dialog_run (GTK_DIALOG (dialog));
 
@@ -1562,6 +1603,9 @@ on_open_location1_activate (GtkButton *button, Totem *totem)
 			mrl = totem_playlist_get_current_mrl (totem->playlist);
 			totem_action_set_mrl_and_play (totem, mrl);
 			g_free (mrl);
+
+			gnome_entry_append_history (GNOME_ENTRY (gentry),
+					TRUE, uri);
 		}
 	}
 
@@ -1735,7 +1779,7 @@ show_controls (Totem *totem, gboolean was_fullscreen)
 	menubar = glade_xml_get_widget (totem->xml, "tmw_menubar");
 	controlbar = glade_xml_get_widget (totem->xml, "tmw_controls_vbox");
 	statusbar = glade_xml_get_widget (totem->xml, "tmw_statusbar");
-	item = glade_xml_get_widget (totem->xml, "trcm_show_controls");
+	item = glade_xml_get_widget (totem->xml_popup, "trcm_show_controls");
 	bvw_vbox = glade_xml_get_widget (totem->xml, "tmw_bvw_vbox");
 	widget = GTK_WIDGET (totem->bvw);
 
@@ -2032,7 +2076,7 @@ commit_hide_skip_to (GtkDialog *dialog, gint response, Totem *totem)
 	{
 		char *msg, *disp;
 
-		disp = gnome_vfs_unescape_string_for_display (totem->mrl);
+		disp = totem_uri_escape_for_display (totem->mrl);
 		msg = g_strdup_printf(_("Totem could not seek in '%s'."), disp);
 		g_free (disp);
 		totem_action_stop (totem);
@@ -2326,7 +2370,7 @@ move_popups (Totem *totem)
 	if (gtk_widget_get_direction (totem->exit_popup) == GTK_TEXT_DIR_RTL)
 	{
 		gtk_window_move (GTK_WINDOW (totem->exit_popup),
-				totem->fullscreen_rect.width - exit_width,
+				totem->fullscreen_rect.x,
 				totem->fullscreen_rect.y);
 		gtk_window_move (GTK_WINDOW (totem->control_popup),
 				totem->fullscreen_rect.width - control_width,
@@ -2334,7 +2378,7 @@ move_popups (Totem *totem)
 				- control_height);
 	} else {
 		gtk_window_move (GTK_WINDOW (totem->exit_popup),
-				totem->fullscreen_rect.x,
+				totem->fullscreen_rect.width - exit_width,
 				totem->fullscreen_rect.y);
 		gtk_window_move (GTK_WINDOW (totem->control_popup),
 				totem->fullscreen_rect.x,
@@ -2361,7 +2405,7 @@ popup_timeout_remove (Totem *totem)
 {
 	if (totem->popup_timeout != 0)
 	{
-		gtk_timeout_remove (totem->popup_timeout);
+		g_source_remove (totem->popup_timeout);
 		totem->popup_timeout = 0;
 	}
 }
@@ -2390,8 +2434,8 @@ on_mouse_click_fullscreen (GtkWidget *widget, Totem *totem)
 {
 	popup_timeout_remove (totem);
 
-	totem->popup_timeout = gtk_timeout_add (2000,
-		(GtkFunction) popup_hide, totem);
+	totem->popup_timeout = g_timeout_add (FULLSCREEN_POPUP_TIMEOUT,
+		(GSourceFunc) popup_hide, totem);
 }
 
 static gboolean
@@ -2422,8 +2466,8 @@ on_video_motion_notify_event (GtkWidget *widget, GdkEventMotion *event,
 	gtk_widget_show_all (totem->control_popup);
 	bacon_video_widget_set_show_cursor (totem->bvw, TRUE);
 
-	totem->popup_timeout = gtk_timeout_add (5000,
-			(GtkFunction) popup_hide, totem);
+	totem->popup_timeout = g_timeout_add (FULLSCREEN_POPUP_TIMEOUT,
+			(GSourceFunc) popup_hide, totem);
 	totem->popup_in_progress = FALSE;
 
 	return FALSE;
@@ -2772,7 +2816,7 @@ update_buttons (Totem *totem)
 	item = glade_xml_get_widget (totem->xml,
 			"tmw_previous_chapter_menu_item");
 	gtk_widget_set_sensitive (item, has_item);
-	item = glade_xml_get_widget (totem->xml, "trcm_previous_chapter");
+	item = glade_xml_get_widget (totem->xml_popup, "trcm_previous_chapter");
 	gtk_widget_set_sensitive (item, has_item);
 
 	/* Next */
@@ -2790,7 +2834,7 @@ update_buttons (Totem *totem)
 	gtk_widget_set_sensitive (item, has_item);
 	item = glade_xml_get_widget (totem->xml, "tmw_next_chapter_menu_item");
 	gtk_widget_set_sensitive (item, has_item);
-	item = glade_xml_get_widget (totem->xml, "trcm_next_chapter");
+	item = glade_xml_get_widget (totem->xml_popup, "trcm_next_chapter");
 	gtk_widget_set_sensitive (item, has_item);
 }
 
@@ -2948,28 +2992,28 @@ totem_callback_connect (Totem *totem)
 			G_CALLBACK (on_show_controls1_activate), totem);
 
 	/* Popup menu */
-	item = glade_xml_get_widget (totem->xml, "trcm_play");
+	item = glade_xml_get_widget (totem->xml_popup, "trcm_play");
 	g_signal_connect (G_OBJECT (item), "activate",
 			G_CALLBACK (on_play1_activate), totem);
-	item = glade_xml_get_widget (totem->xml, "trcm_next_chapter");
+	item = glade_xml_get_widget (totem->xml_popup, "trcm_next_chapter");
 	g_signal_connect (G_OBJECT (item), "activate",
 			G_CALLBACK (on_next_button_clicked), totem);
-	item = glade_xml_get_widget (totem->xml, "trcm_previous_chapter");
+	item = glade_xml_get_widget (totem->xml_popup, "trcm_previous_chapter");
 	g_signal_connect (G_OBJECT (item), "activate",
 			G_CALLBACK (on_previous_button_clicked), totem);
-	item = glade_xml_get_widget (totem->xml, "trcm_skip_forward");
+	item = glade_xml_get_widget (totem->xml_popup, "trcm_skip_forward");
 	g_signal_connect (G_OBJECT (item), "activate",
 			G_CALLBACK (on_skip_forward1_activate), totem);
-	item = glade_xml_get_widget (totem->xml, "trcm_skip_backwards");
+	item = glade_xml_get_widget (totem->xml_popup, "trcm_skip_backwards");
 	g_signal_connect (G_OBJECT (item), "activate",
 			G_CALLBACK (on_skip_backwards1_activate), totem);
-	item = glade_xml_get_widget (totem->xml, "trcm_volume_up");
+	item = glade_xml_get_widget (totem->xml_popup, "trcm_volume_up");
 	g_signal_connect (G_OBJECT (item), "activate",
 			G_CALLBACK (on_volume_up1_activate), totem);
-	item = glade_xml_get_widget (totem->xml, "trcm_volume_down");
+	item = glade_xml_get_widget (totem->xml_popup, "trcm_volume_down");
 	g_signal_connect (G_OBJECT (item), "activate",
 			G_CALLBACK (on_volume_down1_activate), totem);
-	item = glade_xml_get_widget (totem->xml, "trcm_show_controls");
+	item = glade_xml_get_widget (totem->xml_popup, "trcm_show_controls");
 	g_signal_connect (G_OBJECT (item), "activate",
 			G_CALLBACK (on_show_controls2_activate), totem);
 
@@ -3000,7 +3044,7 @@ totem_callback_connect (Totem *totem)
 			G_CALLBACK (drop_playlist_cb), totem);
 	gtk_drag_dest_set (item, GTK_DEST_DEFAULT_ALL,
 			target_table, G_N_ELEMENTS (target_table),
-			GDK_ACTION_COPY);
+			GDK_ACTION_COPY | GDK_ACTION_MOVE);
 
 	/* Main Window */
 	g_signal_connect (G_OBJECT (totem->win), "delete-event",
@@ -3169,7 +3213,7 @@ totem_callback_connect (Totem *totem)
 	totem_set_default_icons (totem);
 
 	/* Update the UI */
-	gtk_timeout_add (600, (GtkFunction) gui_update_cb, totem);
+	g_timeout_add (600, (GSourceFunc) gui_update_cb, totem);
 }
 
 static void
@@ -3216,6 +3260,10 @@ video_widget_create (Totem *totem)
 			"eos",
 			G_CALLBACK (on_eos_event),
 			totem);
+	g_signal_connect (G_OBJECT (totem->bvw),
+			"got-redirect",
+			G_CALLBACK (on_got_redirect),
+			totem);
 	g_signal_connect (G_OBJECT(totem->bvw),
 			"title-change",
 			G_CALLBACK (on_title_change_event),
@@ -3258,7 +3306,7 @@ video_widget_create (Totem *totem)
 			G_CALLBACK (drop_video_cb), totem);
 	gtk_drag_dest_set (GTK_WIDGET (totem->bvw), GTK_DEST_DEFAULT_ALL,
 			target_table, G_N_ELEMENTS (target_table),
-			GDK_ACTION_COPY);
+			GDK_ACTION_COPY | GDK_ACTION_MOVE);
 
 	g_signal_connect (G_OBJECT (totem->bvw), "drag_data_get",
 			G_CALLBACK (drag_video_cb), totem);
@@ -3369,16 +3417,14 @@ main (int argc, char **argv)
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
-	g_set_application_name (_("Totem Movie Player"));
-
 	if (XInitThreads () == 0)
 	{
 		gtk_init (&argc, &argv);
+		g_set_application_name (_("Totem Movie Player"));
 		totem_action_error_and_exit (_("Could not initialise the thread-safe libraries."), _("Verify your system installation. Totem will now exit."), NULL);
 	}
 
 	g_thread_init (NULL);
-	gdk_threads_init ();
 
 #ifdef HAVE_GTK_ONLY
 	gtk_init (&argc, &argv);
@@ -3391,6 +3437,7 @@ main (int argc, char **argv)
 			GNOME_PARAM_NONE);
 #endif /* HAVE_GTK_ONLY */
 
+	g_set_application_name (_("Totem Movie Player"));
 	gnome_vfs_init ();
 
 	if ((gc = gconf_client_get_default ()) == NULL)
@@ -3422,10 +3469,15 @@ main (int argc, char **argv)
 	totem->gc = gc;
 
 	/* Main window */
-	totem->xml = totem_interface_load ("totem.glade", "main window",
+	totem->xml = totem_interface_load ("totem.glade", _("main window"),
 			TRUE, NULL);
 	if (totem->xml == NULL)
 		totem_action_exit (NULL);
+	totem->xml_popup = totem_interface_load ("popup.glade",
+			_("video popup menu"),
+			TRUE, NULL);
+	if (totem->xml_popup == NULL)
+		totem_action_exit (NULL);
 
 	totem->win = glade_xml_get_widget (totem->xml, "totem_main_window");
 	filename = totem_interface_get_full_path ("media-player-48.png");
